%% Non-linear Fit 

% by Alexander Goessling
% Photonics Lab, Muenster University of Applied Sciences
% date: 10/12/2008

%  This is a demonstration of the optimal fitting of a non-linear function to a
%  set of data.  It uses FMINSEARCH, an implementation of the Nelder-Mead simplex
%  (direct search) algorithm, to minimize a nonlinear function of several
%  variables. 
%
%  This program is based on the demo fitdemo.m as provied by MATHSWORKS.
%  Alternatives: Statistics Toolbox
%
% We use this program for fitting Inensity profiles of a single slit
%
% Implemented functions:
% - function fval = single_slit(lambda,t)
%   (diffraction far-field intensity of a slit)
% - function fval = single_slit_fresnel(lambda,t)
%   (diffraction near-field intensity of a slit)

%% Main Program 
function nonlinfit

    % Tabula rasa
    clear all;
    close all;
    clc;
    
%% User input

% +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
% ++++++++++++++++++++++ Change below +++++++++++++++++++++++++++++
    
    % EXAMPLE: Near-field fitting
    
    % filename for input data
    %filein='einzelspalt_C_nahfeld_103mm.lvm'; % example near field
    filein='einzelspalt_C_nahfeld_40mm.lvm'; % example near field
    range=[940,1060];

    % filename for output data 
    fileout_graph = 'nlinfit_test.png';
    fileout_data = 'nlinfit_test.txt';

    % Plot labels 
    xlabeltext='x (pixel)';
    ylabeltext='I (arb. u.)';
    titletext='Single Slit';

    % Plot options (fs - font size, fn - font name, lw - line width)
    fs=12; fn='times'; lw=1;
    
    % Fit options
    % Initial parameters
    % [Distance (m); Slit Width (m); Shift (pixel); Instensity0 (pixel); Intensity offest (pixel)]
    start = [35000;460;1000;4000;100];
    % Tolerance, maximum iterations
    options = optimset('TolX',0.00001,'MaxIter',5000);
    
    % IMPORTANT: Please also change the function FITFUNCT below
    
% +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    
%     %EXAMPLE: Far-field fitting    
%     
%     %filename for input data
%     filein='einzelspalt_C_fernfeld.lvm'; % example far field
%     range=[1,2048];
% 
%     %filename for output data 
%     fileout_graph = 'nlinfit_test.png';
%     fileout_data = 'nlinfit_test.txt';
% 
%     %Plot labels 
%     xlabeltext='x (pixel)';
%     ylabeltext='I (arb. u.)';
%     titletext='Single Slit';
% 
%     %Plot options (fs - font size, fn - font name, lw - line width)
%     fs=12; fn='times'; lw=1;
%     
%     %Fit options
%     %Initial parameters
%     start = [120;1000;4000;100];
%     %Tolerance, maximum iterations
%     options = optimset('TolX',0.00001,'MaxIter',5000);
%     
%      % IMPORTANT: Please also change the function FITFUNCT below
    
% ++++++++++++++++++++++ Change above +++++++++++++++++++++++++++++
% +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

%% Import data
    % data separated with tab = \t
    datain=dlmread(filein,'\t');

    % first row of input data = t, second row of input data = y
    t = datain(range(1):range(2),1);
    y = datain(range(1):range(2),2);

%% Plot data
    figure(1);
    subplot(1,3,1:2);
    plot(t,y,'ro--','LineWidth',lw); hold on; 
    set(gca,'fontsize',fs,'fontname',fn);
    xlabel(xlabeltext);
    ylabel(ylabeltext);
    title(titletext);
    xlim([range(1) range(2)]);

%% Fit
% Make a guess for initial estimate of lambda (start) and invoke FMINSEARCH.  It
% minimizes the error returned from CHISQRD by adjusting lambda.  It returns the
% final value of lambda. 
   
    % Minimization
    disp('Fitting ....');
    [estimated_lambda,fval,exitflag,output] = fminsearch(@(x)chisqrd(x,t,y),start);
    
    % Check whether fit was successful (exitflag=1)    
    switch exitflag
        case 1
            disp(sprintf('Iterations: %d',output.iterations));
            disp('Finished. Fit converged to a solution.');
            
            % Determine the parameter standard errors (variance-covariance matrix)
            covar=covarmatrix(estimated_lambda,t,y,0.001);
            
            % redchisqr = fminsearch(@(u)(gammainc(u/2,length(start)/2)-0.95),1.2,options);
            redchisqr = fval/(length(t)-length(estimated_lambda)); % red chi = chi/(datapoints - number of parameters)
            for i=1:length(estimated_lambda)
                del_estimated_lambda(i)=(covar(i,i)*redchisqr)^(0.5);
            end

            % Plot Result
            tplot=linspace(range(1),range(2),500);
            plotfcn(estimated_lambda,tplot);
            
            % Print fit infos
            subplot(1,3,3);
            set(gca,'fontsize',fs,'fontname',fn)
            axis off
            incy=0.05;
            y0=0.95;
            x0=0.0;
            title('Fit Infos');
            text(x0,y0,['Data points = ',num2str(length(t))],'fontsize',fs,'fontname',fn);
            text(x0,y0-1*incy,['Reduced \chi = ',num2str(redchisqr,'%9.5g')],'fontsize',fs,'fontname',fn);
            text(x0,y0-2*incy,['Model = ',titletext],'fontsize',fs,'fontname',fn);
            text(x0,y0-3.5*incy,['Parameters:'],'fontsize',fs,'fontname',fn);
            for i=1:length(estimated_lambda)
                text(x0,y0-3.5*incy-i*incy,['\lambda(',num2str(i),') = ',num2str(estimated_lambda(i),'%9.5g'), ' \pm ',num2str(del_estimated_lambda(i),'%9.5g')],'fontsize',fs,'fontname',fn)
            end
            
            % Export graph to file
            print('-dpng', '-r300',fileout_graph);
            
            % Export data to file          
            dlmwrite(fileout_data, [t y fitfunct(estimated_lambda,t)], '\t')
            
        case 0
            disp('Maximum number of iterations reached.');
        case -1
            disp('Algorithm was terminated by the output function.');
    end
end % nonlinfit

%% Functions

%% (i) fit function with parameters lambda 

% +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
% ++++++++++++++++++++++ Change below +++++++++++++++++++++++++++++

function fitdata = fitfunct(lambda,t)
    % put in the name of your fitting function (e.g. fitdata=dummy(lambda,t))
    % fitdata = single_slit(lambda,t);
    fitdata = single_slit_fresnel(lambda,t);
end

% single slit, far field 
function fval = single_slit(lambda,t)

    % adapt the parameters to your experimental setting
    wl=0.633;                       % wavelength /m
    z=0.85e6;                       % distance /m
    pixel=14;                       % pixel size /m

    % fit parameters   
    bs1=lambda(1);                  % slit width /m
    shiftx=lambda(2);               % shfit in x direction /pixel
    I0=lambda(3);                   % intensity @ x2=0
    Ioffset=lambda(4);              % intensity offest
    
    % fit function
    % Note: elementwise mathematical operations, e.g. multiplication,
    % of vectors/matrices is implemnte by a "." before the operation, e.g. ".*" for the
    % multiplication.
    fval = Ioffset+I0.*(sin(pixel/wl/z*pi* bs1.*(t-shiftx)).*(pixel/wl/z*pi* bs1.*(t-shiftx)).^(-1)).^2;
end

% ++++++++++++++++++++++ Change above +++++++++++++++++++++++++++++
% +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


% single slit, near field (Fresnel integral)
function fval = single_slit_fresnel(lambda,t)
    
    % adapt the parameters to your experimental setting
    wl=0.633;                       % wavelength / m
    k=2*pi/wl;                      % wavevector
    pixel=14;                       % pixel size /m
    
    % fit parameters
    z=lambda(1);                    % distance /m
    bs1=lambda(2);                  % slit width /m
    shiftx=lambda(3);               % shfit in x direction /pixel
    I0=lambda(4);                   % intensity @ x2=0
    Ioffset=lambda(5);              % intensity offest
    
    %F=bs1^2/wl/z;                  % Frensel number

    N=500;                          % sampling
    x1=linspace(-bs1/2,bs1/2,N);
    x2=t;
    step=bs1/N;

    E0=ones(1,length(x1));          % starting electric field at z=0
    Em1=zeros(size(x2));

    for run=1:length(x2)            %calculate fresnel propagation integral (hodgson)
        Em1(run)=sqrt(2*pi*z/(-i*k))*exp(-i*k*z)/(i*wl*z)*sum(E0.*exp((-i*k*(x1-pixel*x2(run)+pixel*shiftx).^2)/(2*z)));
    end

    Im1=Em1.*conj(Em1);             %calculate intensity
    fval=I0*Im1/max(Im1)+Ioffset;   %normalize intensity
end

% +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
% ++++++++++++++++++++++ Add below ++++++++++++++++++++++++++++++++

% new function dummy (=linear function)
function fval = dummy(lambda,t)
    % fit function
    % Note: elementwise mathematical operations, e.g. multiplication,
    % of vectors/matrices is implemnte by a "." before the operation, e.g. ".*" for the
    % multiplication.
    fval = lambda(1)+lambda(2)*t;
end

% ++++++++++++++++++++++ Add above +---++++++++++++++++++++++++++++
% +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

%% (ii) err function gives chi 

function err = chisqrd(lambda,t,y)
    fitdata = fitfunct(lambda,t);
    err = sum((fitdata-y).^2);
end

%% (iii) plot final result 

function plotfcn(lambda,t)
    fitdata = fitfunct(lambda,t);
    lw=2;
    plot(t,fitdata,'-','LineWidth',lw);
    legend('measured data','fitted data');
end

%% (iv) variance-covariance matrix = inverse Hessian matrix, we use a weighting = 1 for every data point

function covar = covarmatrix(lambda,t,y,step)
for i=1:length(lambda)
    for j=1:length(lambda)
        lambda_i=lambda;
        lambda_j=lambda;
        lambda_i(i)=lambda(i)+lambda(i)*step;
        lambda_j(j)=lambda(j)+lambda(j)*step;
        hess(i,j) = sum((1/(step^2*lambda(i)*lambda(j))).*(fitfunct(lambda_i,t)-fitfunct(lambda,t)).*((fitfunct(lambda_j,t)-fitfunct(lambda,t))));
    end
end
covar=pinv(hess);
end

